@@ -1,11 +1,13 @@
 use core::time::Duration;
 use std::fs::File;
 use std::path::Path;
+use std::sync::Arc;
 
 use clap::{AppSettings, Parser};
 use parquet::file::reader::{ChunkReader, FileReader, SerializedFileReader};
 use parquet::record::reader::RowIter;
 use parquet::schema::printer::print_schema;
+use parquet::schema::types::Type as SchemaType;
 use rusoto_core::Region;
 use url::Url;
 
@@ -46,12 +48,45 @@ fn output_thrift_schema<R: 'static + ChunkReader>(file_reader: &SerializedFileRe
     );
 }
 
+fn get_projection<R: 'static + ChunkReader>(
+    file_reader: &SerializedFileReader<R>,
+    column_names: Option<String>,
+) -> Option<SchemaType> {
+    match column_names {
+        Some(names) => {
+            let parquet_metadata = file_reader.metadata();
+            let schema = parquet_metadata.file_metadata().schema();
+            let column_names = names.split(",");
+            let mut fields: Vec<Arc<SchemaType>> = vec![];
+
+            for column_name in column_names {
+                let found = schema
+                    .get_fields()
+                    .as_ref()
+                    .iter()
+                    .find(|field| field.name().eq(column_name));
+
+                match found {
+                    Some(field) => fields.push(field.clone()),
+                    None => panic!("Column not found ({})", column_name),
+                }
+            }
+            return Some(SchemaType::GroupType {
+                basic_info: schema.get_basic_info().clone(),
+                fields,
+            });
+        }
+        None => None,
+    }
+}
+
 async fn print_json_from(
     source: Source,
     offset: u32,
     limit: i32,
     should_output_schema: bool,
     timeout: Duration,
+    column_names: Option<String>,
 ) {
     match source {
         Source::File(path) => {
@@ -61,7 +96,8 @@ async fn print_json_from(
             if should_output_schema {
                 output_thrift_schema(&file_reader);
             } else {
-                output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+                let projection = get_projection(&file_reader, column_names);
+                output_rows(file_reader.get_row_iter(projection).unwrap(), offset, limit);
             }
         }
         Source::Http(url_str) => {
@@ -74,7 +110,8 @@ async fn print_json_from(
                 if should_output_schema {
                     output_thrift_schema(&file_reader);
                 } else {
-                    output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+                    let projection = get_projection(&file_reader, column_names);
+                    output_rows(file_reader.get_row_iter(projection).unwrap(), offset, limit);
                 }
             });
             blocking_task.await.unwrap();
@@ -97,7 +134,8 @@ async fn print_json_from(
                 if should_output_schema {
                     output_thrift_schema(&file_reader);
                 } else {
-                    output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+                    let projection = get_projection(&file_reader, column_names);
+                    output_rows(file_reader.get_row_iter(projection).unwrap(), offset, limit);
                 }
             });
             blocking_task.await.unwrap();
@@ -124,9 +162,13 @@ struct Cli {
     #[clap(default_value_t = 60, short, long, parse(try_from_str))]
     timeout: u16,
 
-    /// Outputs thrift schema first
+    /// Outputs thrift schema only
     #[clap(short, long)]
     schema_output: Option<bool>,
+
+    /// Select columns by name (comma,separated)
+    #[clap(short, long)]
+    columns: Option<String>,
 }
 
 #[tokio::main]
@@ -138,6 +180,7 @@ async fn main() {
     let limit: i32 = cli.limit.unwrap_or(-1);
     let timeout = Duration::from_secs(cli.timeout.into());
     let file = cli.file;
+    let column_names = cli.columns;
 
     if file.as_str().starts_with("s3://") {
         print_json_from(
@@ -146,6 +189,7 @@ async fn main() {
             limit,
             output_thrift_schema,
             timeout,
+            column_names,
         )
         .await;
     } else if file.as_str().starts_with("http") {
@@ -155,6 +199,7 @@ async fn main() {
             limit,
             output_thrift_schema,
             timeout,
+            column_names,
         )
         .await;
     } else {
@@ -164,6 +209,7 @@ async fn main() {
             limit,
             output_thrift_schema,
             timeout,
+            column_names,
         )
         .await;
     }
